local addonName = ...;
local UnitTracker = ArenaLiveCore:GetHandler("UnitTracker");

local function LockFrame(frame)
	
	if ( frame.handlerList.healthBar ) then
		frame.healthBar.lockValues = true;
		frame.healthBar.lockColour = true;
	end
	
	if ( frame.handlerList.powerBar ) then
		frame.powerBar.lockValues = true;
		frame.powerBar.lockColour = true;
	end
	
	-- For some reason, auras do not reset correctly otherwise:
	frame.auraFrame:Reset();
end

local function UnlockFrame(frame)

	if ( frame.handlerList.healthBar ) then
		frame.healthBar.lockValues = nil;
		frame.healthBar.lockColour = nil;
		frame.healthBar:Update();
	end
	
	if ( frame.handlerList.powerBar ) then
		frame.powerBar.lockValues = nil;
		frame.powerBar.lockColour = nil;
		frame.powerBar:Update();
	end

end

local function CheckUnit(self)
	local id = self:GetID();
	local gameMode = ArenaLiveCore:GetDBEntry(addonName, "GameMode");
	local checkName = ArenaLiveCore:GetDBEntry(addonName, "Teams/"..self.side.."/Member"..id);
	local newUnit = ArenaLiveSpectator.UnitsByNameTable[checkName];
	
	if ( id > gameMode ) then
		self:ResetUnit();
		self:Reset();	
	elseif ( self.unit ) then
		if ( newUnit and self.unit ~= newUnit ) then
			UnlockFrame(self);
			self:SetUnit(ArenaLiveSpectator.UnitsByNameTable[checkName]);
		elseif ( not newUnit ) then
			self:ResetUnit();
			self:Reset();
		end
	elseif ( checkName ) then
		if ( ArenaLiveSpectator.UnitsByNameTable[checkName] ) then
			UnlockFrame(self);
			self:SetUnit(ArenaLiveSpectator.UnitsByNameTable[checkName]);
		end
	end

	-- Update cooldown tracker:
	local tracker = _G["ALSpec_CooldownTracker"..self.side];
	tracker:UpdateTrackedUnits();

end

function UpdateVisibility(self)
	local id = self:GetID();
	local gameMode = ArenaLiveCore:GetDBEntry(addonName, "GameMode");
	
	if ( id <= gameMode ) then
		self:SetAttribute("alforceshow", true);
		self:CheckUnit();
	else
		self:SetAttribute("alforceshow", nil);
		self:ResetUnit();
		self:Reset();
	end

end

local function OnEvent(self, event, ...)
	
	if ( not self.unit ) then
		return;
	end
	
	if ( event == "ARENA_OPPONENT_UPDATE" ) then
		local unit, state = ...;
		
		if ( unit == self.unit )then
			if ( state == "seen" ) then
				if ( UnitExists(unit) ) then
					UnlockFrame(self);
					self:Update();
				end
			elseif ( state == "destroyed" ) then
				UnlockFrame(self);
				self:Update();
			elseif ( state == "cleared" ) then
				UnlockFrame(self);
				frame:Reset();
			elseif ( state == "unseen" ) then
				LockFrame(self);
			end
		end
	end
end

function ArenaLiveSpectator:InitialiseSideFrame(frame, side)
	local prefix = frame:GetName();
	
	frame.side = side;

	-- Add functions to check if the shown unit is still correct:
	frame.CheckUnit = CheckUnit;
	frame.UpdateVisibility = UpdateVisibility;
	
	ArenaLiveCore:AddFrame(frame, "UnitFrame", addonName, "SideFrames", menufunc, true);
	frame:SetAttribute("alframelock", true);
	ArenaLiveCore:AddFrame(_G[prefix.."Name"], "NameText", frame);
	ArenaLiveCore:AddFrame(_G[prefix.."Portrait"], "Portrait", _G[prefix.."PortraitTexture"], _G[prefix.."Portrait3D"], frame);
	ArenaLiveCore:AddFrame(_G[prefix.."HealthBar"], "HealthBar", frame, false);
	ArenaLiveCore:AddFrame (_G[prefix.."HealthBarText"], "StatusBarText", "HealthBarText", nil, nil, true, frame);
	ArenaLiveCore:AddFrame(_G[prefix.."PowerBar"], "PowerBar", frame, false);
	ArenaLiveCore:AddFrame (_G[prefix.."PowerBarText"], "StatusBarText", "PowerBarText", nil, nil, nil, frame);
	ArenaLiveCore:AddFrame(_G[prefix.."Icon1"], "Icon", _G[prefix.."Icon1Texture"], _G[prefix.."Icon1Cooldown"], _G[prefix.."Icon1CooldownText"], frame);
	ArenaLiveCore:AddFrame(_G[prefix.."Icon2"], "Icon", _G[prefix.."Icon2Texture"], _G[prefix.."Icon2Cooldown"], _G[prefix.."Icon2CooldownText"], frame);
	ArenaLiveCore:AddFrame(_G[prefix.."PortraitOverlay"], "CCIndicator", _G[prefix.."PortraitOverlayIcon"], _G[prefix.."PortraitOverlayCooldown"], _G[prefix.."PortraitOverlayCooldownText"], frame);
	ArenaLiveCore:AddFrame (_G[prefix.."AuraFrame"], "Aura", _G[prefix.."AuraFrameBuffFrame"], _G[prefix.."AuraFrameDebuffFrame"], "ArenaLive_BuffTemplate", "ArenaLive_DebuffTemplate", frame);
	ArenaLiveCore:AddFrame(_G[prefix.."AbsorbBar"], "AbsorbBar", _G[prefix.."AbsorbBarOverlay"], 32, _G[prefix.."AbsorbBarFullHPIndicator"], frame);
	ArenaLiveCore:AddFrame(_G[prefix.."CastHistory"], "CastHistory", "ArenaLive_CastHistoryIconTemplate", frame);
	ArenaLiveCore:AddFrame(_G[prefix.."SpiritHealerFrame"], "SpiritHealer", _G[prefix.."SpiritHealerFrameTexture"], _G[prefix.."SpiritHealerFrameFlash"], 5, frame);
	
	frame:Reset();
	frame:CheckUnit();
	frame:UpdateVisibility();
	frame:RegisterEvent("ARENA_OPPONENT_UPDATE");
	frame:SetScript("OnEvent", OnEvent);
end

function ArenaLiveSpectator:CheckAllSideFrameUnits()
	for i = 1, 3 do
		local frame = _G["ALSpec_LeftFrame"..i];
		frame:CheckUnit();
		
		frame = _G["ALSpec_RightFrame"..i];
		frame:CheckUnit();
	end
end

function ArenaLiveSpectator:UpdateAllSideFramesVisibility()
	for i = 1, 3 do
		local frame = _G["ALSpec_LeftFrame"..i];
		frame:UpdateVisibility();
		
		frame = _G["ALSpec_RightFrame"..i];
		frame:UpdateVisibility();
	end
end